#! /usr/bin/perl
################################################################################
## NAME: kontroll-stats-gather-5.0.x_linux-x86.pl
## VERSION: 2.0.1
## DATE: 2008-12-01
## AUTHOR: Matt Reid
## WEBSITE: http://kontrollsoft.com
## EMAIL: kontact@kontrollsoft.com
## LICENSE: PLEASE REFER TO THE LICENSE.txt file bundled with this software release
## to understand your right and options for using this software.
## Copyright 2008 Kontrollsoft LLC
## All rights reserved.
################################################################################

use strict;
use warnings;
use DBI;
use XML::Parser;
use XML::SimpleObject;
use Fcntl; 
use POSIX qw(strftime);
use Time::HiRes qw(gettimeofday tv_interval);

my $config = "../config.cfg";
my $mysql_r_user = undef;
my $mysql_r_pass = undef;
my $mysql_r_db = undef;
my $mysql_r_host = undef;
my $mysql_w_user = undef;
my $mysql_w_pass = undef;
my $mysql_w_db = undef;
my $mysql_w_host = undef;
my $error_log = undef;
my $debug_log = undef;
my $datetime =  strftime "%Y-%m-%d %H:%M:%S", localtime;
my $h = undef;

## begin long list of variable definitions
my %varlist = (
    'server_list_id' => '0',
    'cnf_file' => '0',
    'os_load_1' => '0',
    'os_load_5' => '0',
    'os_load_15' => '0',
    'os_mem_total' => '0',
    'os_mem_used' => '0',
    'os_swap_total' => '0',
    'os_swap_free' => '0',
    'os_cpu_user' => '0',
    'os_cpu_system' => '0',
    'os_cpu_idle' => '0',
    'queries_per_second' => '0',
    'num_schema' => '0',
    'num_tables' => '0',
    'num_connections' => '0',
    'length_data' => '0',
    'length_index' => '0',
    'engine_count_innodb' => '0',
    'engine_count_myisam' => '0',
    'engine_myisam_size_data' => '0',
    'engine_myisam_size_index' => '0',
    'engine_innodb_size_data' => '0',
    'engine_innodb_size_index' => '0',
    'auto_increment_increment' => '0',
    'auto_increment_offset' => '0',
    'automatic_sp_privileges' => '0',
    'back_log' => '0',
    'basedir' => '0',
    'binlog_cache_size' => '0',
    'bulk_insert_buffer_size' => '0',
    'character_set_client' => '0',
    'character_set_connection' => '0',
    'character_set_database' => '0',
    'character_set_filesystem' => '0',
    'character_set_results' => '0',
    'character_set_server' => '0',
    'character_set_system' => '0',
    'character_sets_dir' => '0',
    'collation_connection' => '0',
    'collation_database' => '0',
    'collation_server' => '0',
    'completion_type' => '0',
    'concurrent_insert' => '0',
    'connect_timeout' => '0',
    'datadir' => '0',
    'date_format' => '0',
    'datetime_format' => '0',
    'default_week_format' => '0',
    'delay_key_write' => '0',
    'delayed_insert_limit' => '0',
    'delayed_insert_timeout' => '0',
    'delayed_queue_size' => '0',
    'div_precision_increment' => '0',
    'keep_files_on_create' => '0',
    'engine_condition_pushdown' => '0',
    'expire_logs_days' => '0',
    'flush' => '0',
    'flush_time' => '0',
    'ft_boolean_syntax' => '0',
    'ft_max_word_len' => '0',
    'ft_min_word_len' => '0',
    'ft_query_expansion_limit' => '0',
    'ft_stopword_file' => '0',
    'group_concat_max_len' => '0',
    'have_archive' => '0',
    'have_bdb' => '0',
    'have_blackhole_engine' => '0',
    'have_compress' => '0',
    'have_crypt' => '0',
    'have_csv' => '0',
    'have_dynamic_loading' => '0',
    'have_example_engine' => '0',
    'have_federated_engine' => '0',
    'have_geometry' => '0',
    'have_innodb' => '0',
    'have_isam' => '0',
    'have_merge_engine' => '0',
    'have_ndbcluster' => '0',
    'have_openssl' => '0',
    'have_ssl' => '0',
    'have_query_cache' => '0',
    'have_raid' => '0',
    'have_rtree_keys' => '0',
    'have_symlink' => '0',
    'hostname' => '0',
    'init_connect' => '0',
    'init_file' => '0',
    'init_slave' => '0',
    'innodb_additional_mem_pool_size' => '0',
    'innodb_autoextend_increment' => '0',
    'innodb_buffer_pool_awe_mem_mb' => '0',
    'innodb_buffer_pool_size' => '0',
    'innodb_checksums' => '0',
    'innodb_commit_concurrency' => '0',
    'innodb_concurrency_tickets' => '0',
    'innodb_data_file_path' => '0',
    'innodb_data_home_dir' => '0',
    'innodb_adaptive_hash_index' => '0',
    'innodb_doublewrite' => '0',
    'innodb_fast_shutdown' => '0',
    'innodb_file_io_threads' => '0',
    'innodb_file_per_table' => '0',
    'innodb_flush_log_at_trx_commit' => '0',
    'innodb_flush_method' => '0',
    'innodb_force_recovery' => '0',
    'innodb_lock_wait_timeout' => '0',
    'innodb_locks_unsafe_for_binlog' => '0',
    'innodb_log_arch_dir' => '0',
    'innodb_log_archive' => '0',
    'innodb_log_buffer_size' => '0',
    'innodb_log_file_size' => '0',
    'innodb_log_files_in_group' => '0',
    'innodb_log_group_home_dir' => '0',
    'innodb_max_dirty_pages_pct' => '0',
    'innodb_max_purge_lag' => '0',
    'innodb_mirrored_log_groups' => '0',
    'innodb_open_files' => '0',
    'innodb_rollback_on_timeout' => '0',
    'innodb_support_xa' => '0',
    'innodb_sync_spin_loops' => '0',
    'innodb_table_locks' => '0',
    'innodb_thread_concurrency' => '0',
    'innodb_thread_sleep_delay' => '0',
    'innodb_read_ahead' => '0',
    'innodb_ibuf_contract_const' => '0',
    'innodb_ibuf_contract_burst' => '0',
    'innodb_buf_flush_const' => '0',
    'innodb_buf_flush_burst' => '0',
    'interactive_timeout' => '0',
    'join_buffer_size' => '0',
    'key_buffer_size' => '0',
    'key_cache_age_threshold' => '0',
    'key_cache_block_size' => '0',
    'key_cache_division_limit' => '0',
    'language' => '0',
    'large_files_support' => '0',
    'large_page_size' => '0',
    'large_pages' => '0',
    'lc_time_names' => '0',
    'license' => '0',
    'local_infile' => '0',
    'locked_in_memory' => '0',
    'log' => '0',
    'log_bin' => '0',
    'log_bin_trust_function_creators' => '0',
    'log_error' => '0',
    'log_queries_not_using_indexes' => '0',
    'log_slave_updates' => '0',
    'log_slow_queries' => '0',
    'log_slow_filter' => '0',
    'log_slow_verbosity' => '0',
    'log_warnings' => '0',
    'long_query_time' => '0',
    'low_priority_updates' => '0',
    'lower_case_file_system' => '0',
    'lower_case_table_names' => '0',
    'max_allowed_packet' => '0',
    'max_binlog_cache_size' => '0',
    'max_binlog_size' => '0',
    'max_connect_errors' => '0',
    'max_connections' => '0',
    'max_delayed_threads' => '0',
    'max_error_count' => '0',
    'max_heap_table_size' => '0',
    'max_insert_delayed_threads' => '0',
    'max_join_size' => '0',
    'max_length_for_sort_data' => '0',
    'max_prepared_stmt_count' => '0',
    'max_relay_log_size' => '0',
    'max_seeks_for_key' => '0',
    'max_sort_length' => '0',
    'max_sp_recursion_depth' => '0',
    'max_tmp_tables' => '0',
    'max_user_connections' => '0',
    'max_write_lock_count' => '0',
    'min_examined_row_limit' => '0',
    'multi_range_count' => '0',
    'myisam_data_pointer_size' => '0',
    'myisam_max_sort_file_size' => '0',
    'myisam_recover_options' => '0',
    'myisam_repair_threads' => '0',
    'myisam_sort_buffer_size' => '0',
    'myisam_stats_method' => '0',
    'net_buffer_length' => '0',
    'net_read_timeout' => '0',
    'net_retry_count' => '0',
    'net_write_timeout' => '0',
    'new' => '0',
    'old_passwords' => '0',
    'open_files_limit' => '0',
    'optimizer_prune_level' => '0',
    'optimizer_search_depth' => '0',
    'pid_file' => '0',
    'port' => '0',
    'preload_buffer_size' => '0',
    'protocol_version' => '0',
    'query_alloc_block_size' => '0',
    'query_cache_limit' => '0',
    'query_cache_min_res_unit' => '0',
    'query_cache_size' => '0',
    'query_cache_type' => '0',
    'query_cache_wlock_invalidate' => '0',
    'query_prealloc_size' => '0',
    'range_alloc_block_size' => '0',
    'log_slow_rate_limit' => '0',
    'read_buffer_size' => '0',
    'read_only' => '0',
    'read_rnd_buffer_size' => '0',
    'relay_log' => '0',
    'relay_log_index' => '0',
    'relay_log_info_file' => '0',
    'relay_log_purge' => '0',
    'relay_log_space_limit' => '0',
    'rpl_recovery_rank' => '0',
    'secure_auth' => '0',
    'secure_file_priv' => '0',
    'server_id' => '0',
    'skip_external_locking' => '0',
    'skip_networking' => '0',
    'skip_show_database' => '0',
    'slave_compressed_protocol' => '0',
    'slave_load_tmpdir' => '0',
    'slave_net_timeout' => '0',
    'slave_skip_errors' => '0',
    'slave_transaction_retries' => '0',
    'slow_launch_time' => '0',
    'socket' => '0',
    'sort_buffer_size' => '0',
    'sql_big_selects' => '0',
    'sql_mode' => '0',
    'sql_notes' => '0',
    'sql_warnings' => '0',
    'ssl_ca' => '0',
    'ssl_capath' => '0',
    'ssl_cert' => '0',
    'ssl_cipher' => '0',
    'ssl_key' => '0',
    'storage_engine' => '0',
    'sync_binlog' => '0',
    'sync_frm' => '0',
    'system_time_zone' => '0',
    'table_cache' => '0',
    'table_lock_wait_timeout' => '0',
    'table_type' => '0',
    'thread_cache_size' => '0',
    'thread_stack' => '0',
    'time_format' => '0',
    'time_zone' => '0',
    'timed_mutexes' => '0',
    'tmp_table_size' => '0',
    'tmpdir' => '0',
    'transaction_alloc_block_size' => '0',
    'transaction_prealloc_size' => '0',
    'tx_isolation' => '0',
    'updatable_views_with_limit' => '0',
    'version' => '0',
    'version_comment' => '0',
    'version_compile_machine' => '0',
    'version_compile_os' => '0',
    'wait_timeout' => '0',
    'Aborted_clients' => '0',
    'Aborted_connects' => '0',
    'Binlog_cache_disk_use' => '0',
    'Binlog_cache_use' => '0',
    'Bytes_received' => '0',
    'Bytes_sent' => '0',
    'Com_admin_commands' => '0',
    'Com_alter_db' => '0',
    'Com_alter_table' => '0',
    'Com_analyze' => '0',
    'Com_backup_table' => '0',
    'Com_begin' => '0',
    'Com_call_procedure' => '0',
    'Com_change_db' => '0',
    'Com_change_master' => '0',
    'Com_check' => '0',
    'Com_checksum' => '0',
    'Com_commit' => '0',
    'Com_create_db' => '0',
    'Com_create_function' => '0',
    'Com_create_index' => '0',
    'Com_create_table' => '0',
    'Com_create_user' => '0',
    'Com_dealloc_sql' => '0',
    'Com_delete' => '0',
    'Com_delete_multi' => '0',
    'Com_do' => '0',
    'Com_drop_db' => '0',
    'Com_drop_function' => '0',
    'Com_drop_index' => '0',
    'Com_drop_table' => '0',
    'Com_drop_user' => '0',
    'Com_execute_sql' => '0',
    'Com_flush' => '0',
    'Com_grant' => '0',
    'Com_ha_close' => '0',
    'Com_ha_open' => '0',
    'Com_ha_read' => '0',
    'Com_help' => '0',
    'Com_insert' => '0',
    'Com_insert_select' => '0',
    'Com_kill' => '0',
    'Com_load' => '0',
    'Com_load_master_data' => '0',
    'Com_load_master_table' => '0',
    'Com_lock_tables' => '0',
    'Com_optimize' => '0',
    'Com_preload_keys' => '0',
    'Com_prepare_sql' => '0',
    'Com_purge' => '0',
    'Com_purge_before_date' => '0',
    'Com_rename_table' => '0',
    'Com_repair' => '0',
    'Com_replace' => '0',
    'Com_replace_select' => '0',
    'Com_reset' => '0',
    'Com_restore_table' => '0',
    'Com_revoke' => '0',
    'Com_revoke_all' => '0',
    'Com_rollback' => '0',
    'Com_savepoint' => '0',
    'Com_select' => '0',
    'Com_set_option' => '0',
    'Com_show_binlog_events' => '0',
    'Com_show_binlogs' => '0',
    'Com_show_charsets' => '0',
    'Com_show_collations' => '0',
    'Com_show_column_types' => '0',
    'Com_show_create_db' => '0',
    'Com_show_create_table' => '0',
    'Com_show_databases' => '0',
    'Com_show_errors' => '0',
    'Com_show_fields' => '0',
    'Com_show_grants' => '0',
    'Com_show_innodb_status' => '0',
    'Com_show_keys' => '0',
    'Com_show_logs' => '0',
    'Com_show_master_status' => '0',
    'Com_show_ndb_status' => '0',
    'Com_show_new_master' => '0',
    'Com_show_open_tables' => '0',
    'Com_show_privileges' => '0',
    'Com_show_processlist' => '0',
    'Com_show_slave_hosts' => '0',
    'Com_show_slave_status' => '0',
    'Com_show_status' => '0',
    'Com_show_storage_engines' => '0',
    'Com_show_tables' => '0',
    'Com_show_triggers' => '0',
    'Com_show_variables' => '0',
    'Com_show_warnings' => '0',
    'Com_slave_start' => '0',
    'Com_slave_stop' => '0',
    'Com_stmt_close' => '0',
    'Com_stmt_execute' => '0',
    'Com_stmt_fetch' => '0',
    'Com_stmt_prepare' => '0',
    'Com_stmt_reset' => '0',
    'Com_stmt_send_long_data' => '0',
    'Com_truncate' => '0',
    'Com_unlock_tables' => '0',
    'Com_update' => '0',
    'Com_update_multi' => '0',
    'Com_xa_commit' => '0',
    'Com_xa_end' => '0',
    'Com_xa_prepare' => '0',
    'Com_xa_recover' => '0',
    'Com_xa_rollback' => '0',
    'Com_xa_start' => '0',
    'Compression' => '0',
    'Connections' => '0',
    'Created_tmp_disk_tables' => '0',
    'Created_tmp_files' => '0',
    'Created_tmp_tables' => '0',
    'Delayed_errors' => '0',
    'Delayed_insert_threads' => '0',
    'Delayed_writes' => '0',
    'Flush_commands' => '0',
    'Handler_commit' => '0',
    'Handler_delete' => '0',
    'Handler_discover' => '0',
    'Handler_prepare' => '0',
    'Handler_read_first' => '0',
    'Handler_read_key' => '0',
    'Handler_read_next' => '0',
    'Handler_read_prev' => '0',
    'Handler_read_rnd' => '0',
    'Handler_read_rnd_next' => '0',
    'Handler_rollback' => '0',
    'Handler_savepoint' => '0',
    'Handler_savepoint_rollback' => '0',
    'Handler_update' => '0',
    'Handler_write' => '0',
    'Innodb_buffer_pool_pages_data' => '0',
    'Innodb_buffer_pool_pages_dirty' => '0',
    'Innodb_buffer_pool_pages_flushed' => '0',
    'Innodb_buffer_pool_pages_free' => '0',
    'Innodb_buffer_pool_pages_misc' => '0',
    'Innodb_buffer_pool_pages_total' => '0',
    'Innodb_buffer_pool_read_ahead_rnd' => '0',
    'Innodb_buffer_pool_read_ahead_seq' => '0',
    'Innodb_buffer_pool_read_requests' => '0',
    'Innodb_buffer_pool_reads' => '0',
    'Innodb_buffer_pool_wait_free' => '0',
    'Innodb_buffer_pool_write_requests' => '0',
    'Innodb_data_fsyncs' => '0',
    'Innodb_data_pending_fsyncs' => '0',
    'Innodb_data_pending_reads' => '0',
    'Innodb_data_pending_writes' => '0',
    'Innodb_data_read' => '0',
    'Innodb_data_reads' => '0',
    'Innodb_data_writes' => '0',
    'Innodb_data_written' => '0',
    'Innodb_dblwr_pages_written' => '0',
    'Innodb_dblwr_writes' => '0',
    'Innodb_log_waits' => '0',
    'Innodb_log_write_requests' => '0',
    'Innodb_log_writes' => '0',
    'Innodb_os_log_fsyncs' => '0',
    'Innodb_os_log_pending_fsyncs' => '0',
    'Innodb_os_log_pending_writes' => '0',
    'Innodb_os_log_written' => '0',
    'Innodb_page_size' => '0',
    'Innodb_pages_created' => '0',
    'Innodb_pages_read' => '0',
    'Innodb_pages_written' => '0',
    'Innodb_row_lock_current_waits' => '0',
    'Innodb_row_lock_time' => '0',
    'Innodb_row_lock_time_avg' => '0',
    'Innodb_row_lock_time_max' => '0',
    'Innodb_row_lock_waits' => '0',
    'Innodb_rows_deleted' => '0',
    'Innodb_rows_inserted' => '0',
    'Innodb_rows_read' => '0',
    'Innodb_rows_updated' => '0',
    'Key_blocks_not_flushed' => '0',
    'Key_blocks_unused' => '0',
    'Key_blocks_used' => '0',
    'Key_read_requests' => '0',
    'Key_reads' => '0',
    'Key_write_requests' => '0',
    'Key_writes' => '0',
    'Last_query_cost' => '0',
    'Max_used_connections' => '0',
    'Not_flushed_delayed_rows' => '0',
    'Open_files' => '0',
    'Open_streams' => '0',
    'Open_tables' => '0',
    'Opened_tables' => '0',
    'Prepared_stmt_count' => '0',
    'Qcache_free_blocks' => '0',
    'Qcache_free_memory' => '0',
    'Qcache_hits' => '0',
    'Qcache_inserts' => '0',
    'Qcache_lowmem_prunes' => '0',
    'Qcache_not_cached' => '0',
    'Qcache_queries_in_cache' => '0',
    'Qcache_total_blocks' => '0',
    'Questions' => '0',
    'Rpl_status' => '0',
    'Select_full_join' => '0',
    'Select_full_range_join' => '0',
    'Select_range' => '0',
    'Select_range_check' => '0',
    'Select_scan' => '0',
    'Slave_open_temp_tables' => '0',
    'Slave_retried_transactions' => '0',
    'Slave_running' => '0',
    'Slow_launch_threads' => '0',
    'Slow_queries' => '0',
    'Sort_merge_passes' => '0',
    'Sort_range' => '0',
    'Sort_rows' => '0',
    'Sort_scan' => '0',
    'Table_locks_immediate' => '0',
    'Table_locks_waited' => '0',
    'Tc_log_max_pages_used' => '0',
    'Tc_log_page_size' => '0',
    'Tc_log_page_waits' => '0',
    'Threads_cached' => '0',
    'Threads_connected' => '0',
    'Threads_created' => '0',
    'Threads_running' => '0',
    'Uptime' => '0',
    'server_snmp_error_code' => '0',
    'server_mysql_error_code' => '0',
    'Slave_SQL_Running' => '0',
    'Slave_IO_Running' => '0',
    'Seconds_Behind_Master' => '0',
    'illegal_global_user' => '0',
    'illegal_grant_user' => '0',
    'illegal_remote_root' => '0',
    'illegal_user_nopass' => '0',
    'illegal_user_noname' => '0',
    'collection_time_elapse' => '0');

sub config_connect {    
    open(DAT, $_[0]) || die ("Could not open config file ($_[0])");
    my @data=<DAT>;
    close(DAT);    
    @data = grep(!/^#/, @data); #remove comment lines
    foreach my $item(@data) { chomp($item);} #remove carriage returns

    $mysql_r_user = $data[0];
    $mysql_r_pass = $data[1];
    $mysql_r_db = $data[2];
    $mysql_r_host = $data[3];
    $mysql_w_user = $data[4];
    $mysql_w_pass = $data[5];
    $mysql_w_db = $data[6];
    $mysql_w_host = $data[7];
    $error_log = $data[8];
    $debug_log = $data[9];

    #remove the starting string                                                                                                                           
    $mysql_r_user =~ s/MYSQL_R_USER=//g;
    $mysql_r_pass =~ s/MYSQL_R_PASS=//g;
    $mysql_r_db =~ s/MYSQL_R_DB=//g;
    $mysql_r_host =~ s/MYSQL_R_HOST=//g;
    $mysql_w_user =~ s/MYSQL_W_USER=//g;
    $mysql_w_pass =~ s/MYSQL_W_PASS=//g;
    $mysql_w_db =~ s/MYSQL_W_DB=//g;
    $mysql_w_host =~ s/MYSQL_W_HOST=//g;
    $error_log =~ s/ERROR_LOG=//g;
    $debug_log =~ s/DEBUG_LOG=//g;
}

sub insert_data {
    my $sql0 = "INSERT INTO `server_statistics` (
`id`, 
`server_list_id`, 
`cnf_file`, 
`os_load_1`, 
`os_load_5`, 
`os_load_15`, 
`os_mem_total`, 
`os_mem_used`, 
`os_swap_total`, 
`os_swap_free`, 
`os_cpu_user`, 
`os_cpu_system`, 
`os_cpu_idle`, 
`queries_per_second`, 
`num_schema`, 
`num_tables`, 
`num_connections`,
`length_data`, 
`length_index`, 
`engine_count_innodb`, 
`engine_count_myisam`, 
`engine_myisam_size_data`,
`engine_myisam_size_index`,
`engine_innodb_size_data`,
`engine_innodb_size_index`,
`auto_increment_increment`,
`auto_increment_offset`,
`automatic_sp_privileges`,
`back_log`,
`basedir`,
`binlog_cache_size`,
`bulk_insert_buffer_size`,
`character_set_client`,
`character_set_connection`,
`character_set_database`,
`character_set_filesystem`,
`character_set_results`,
`character_set_server`,
`character_set_system`,
`character_sets_dir`,
`collation_connection`,
`collation_database`,
`collation_server`,
`completion_type`,
`concurrent_insert`,
`connect_timeout`,
`datadir`,
`date_format`,
`datetime_format`,
`default_week_format`,
`delay_key_write`,
`delayed_insert_limit`,
`delayed_insert_timeout`,
`delayed_queue_size`,
`div_precision_increment`,
`keep_files_on_create`,
`engine_condition_pushdown`,
`expire_logs_days`,
`flush`,
`flush_time`,
`ft_boolean_syntax`,
`ft_max_word_len`,
`ft_min_word_len`,
`ft_query_expansion_limit`,
`ft_stopword_file`,
`group_concat_max_len`,
`have_archive`,
`have_bdb`,
`have_blackhole_engine`,
`have_compress`,
`have_crypt`,
`have_csv`,
`have_dynamic_loading`,
`have_example_engine`,
`have_federated_engine`,
`have_geometry`,
`have_innodb`,
`have_isam`,
`have_merge_engine`,
`have_ndbcluster`,
`have_openssl`,
`have_ssl`,
`have_query_cache`,
`have_raid`,
`have_rtree_keys`,
`have_symlink`,
`hostname`,
`init_connect`,
`init_file`,
`init_slave`,
`innodb_additional_mem_pool_size`,
`innodb_autoextend_increment`,
`innodb_buffer_pool_awe_mem_mb`,
`innodb_buffer_pool_size`,
`innodb_checksums`,
`innodb_commit_concurrency`,
`innodb_concurrency_tickets`,
`innodb_data_file_path`,
`innodb_data_home_dir`,
`innodb_adaptive_hash_index`,
`innodb_doublewrite`,
`innodb_fast_shutdown`,
`innodb_file_io_threads`,
`innodb_file_per_table`,
`innodb_flush_log_at_trx_commit`,
`innodb_flush_method`,
`innodb_force_recovery`,
`innodb_lock_wait_timeout`,
`innodb_locks_unsafe_for_binlog`,
`innodb_log_arch_dir`,
`innodb_log_archive`,
`innodb_log_buffer_size`,
`innodb_log_file_size`,
`innodb_log_files_in_group`,
`innodb_log_group_home_dir`,
`innodb_max_dirty_pages_pct`,
`innodb_max_purge_lag`,
`innodb_mirrored_log_groups`,
`innodb_open_files`,
`innodb_rollback_on_timeout`,
`innodb_support_xa`,
`innodb_sync_spin_loops`,
`innodb_table_locks`,
`innodb_thread_concurrency`,
`innodb_thread_sleep_delay`,
`innodb_read_ahead`,
`innodb_ibuf_contract_const`,
`innodb_ibuf_contract_burst`,
`innodb_buf_flush_const`,
`innodb_buf_flush_burst`,
`interactive_timeout`,
`join_buffer_size`,
`key_buffer_size`,
`key_cache_age_threshold`,
`key_cache_block_size`,
`key_cache_division_limit`,
`language`,
`large_files_support`,
`large_page_size`,
`large_pages`,
`lc_time_names`,
`license`,
`local_infile`,
`locked_in_memory`,
`log`,
`log_bin`,
`log_bin_trust_function_creators`,
`log_error`,
`log_queries_not_using_indexes`,
`log_slave_updates`,
`log_slow_queries`,
`log_slow_filter`,
`log_slow_verbosity`,
`log_warnings`,
`long_query_time`,
`low_priority_updates`,
`lower_case_file_system`,
`lower_case_table_names`,
`max_allowed_packet`,
`max_binlog_cache_size`,
`max_binlog_size`,
`max_connect_errors`,
`max_connections`,
`max_delayed_threads`,
`max_error_count`,
`max_heap_table_size`,
`max_insert_delayed_threads`,
`max_join_size`,
`max_length_for_sort_data`,
`max_prepared_stmt_count`,
`max_relay_log_size`,
`max_seeks_for_key`,
`max_sort_length`,
`max_sp_recursion_depth`,
`max_tmp_tables`,
`max_user_connections`,
`max_write_lock_count`,
`min_examined_row_limit`,
`multi_range_count`,
`myisam_data_pointer_size`,
`myisam_max_sort_file_size`,
`myisam_recover_options`,
`myisam_repair_threads`,
`myisam_sort_buffer_size`,
`myisam_stats_method`,
`net_buffer_length`,
`net_read_timeout`,
`net_retry_count`,
`net_write_timeout`,
`new`,
`old_passwords`,
`open_files_limit`,
`optimizer_prune_level`,
`optimizer_search_depth`,
`pid_file`,
`port`,
`preload_buffer_size`,
`protocol_version`,
`query_alloc_block_size`,
`query_cache_limit`,
`query_cache_min_res_unit`,
`query_cache_size`,
`query_cache_type`,
`query_cache_wlock_invalidate`,
`query_prealloc_size`,
`range_alloc_block_size`,
`log_slow_rate_limit`,
`read_buffer_size`,
`read_only`,
`read_rnd_buffer_size`,
`relay_log`,
`relay_log_index`,
`relay_log_info_file`,
`relay_log_purge`,
`relay_log_space_limit`,
`rpl_recovery_rank`,
`secure_auth`,
`secure_file_priv`,
`server_id`,
`skip_external_locking`,
`skip_networking`,
`skip_show_database`,
`slave_compressed_protocol`,
`slave_load_tmpdir`,
`slave_net_timeout`,
`slave_skip_errors`,
`slave_transaction_retries`,
`slow_launch_time`,
`socket`,
`sort_buffer_size`,
`sql_big_selects`,
`sql_mode`,
`sql_notes`,
`sql_warnings`,
`ssl_ca`,
`ssl_capath`,
`ssl_cert`,
`ssl_cipher`,
`ssl_key`,
`storage_engine`,
`sync_binlog`,
`sync_frm`,
`system_time_zone`,
`table_cache`,
`table_lock_wait_timeout`,
`table_type`,
`thread_cache_size`,
`thread_stack`,
`time_format`,
`time_zone`,
`timed_mutexes`,
`tmp_table_size`,
`tmpdir`,
`transaction_alloc_block_size`,
`transaction_prealloc_size`,
`tx_isolation`,
`updatable_views_with_limit`,
`version`,
`version_comment`,
`version_compile_machine`,
`version_compile_os`,
`wait_timeout`,
`Aborted_clients`,
`Aborted_connects`,
`Binlog_cache_disk_use`,
`Binlog_cache_use`,
`Bytes_received`,
`Bytes_sent`,
`Com_admin_commands`,
`Com_alter_db`,
`Com_alter_table`,
`Com_analyze`,
`Com_backup_table`,
`Com_begin`,
`Com_call_procedure`,
`Com_change_db`,
`Com_change_master`,
`Com_check`,
`Com_checksum`,
`Com_commit`,
`Com_create_db`,
`Com_create_function`,
`Com_create_index`,
`Com_create_table`,
`Com_create_user`,
`Com_dealloc_sql`,
`Com_delete`,
`Com_delete_multi`,
`Com_do`,
`Com_drop_db`,
`Com_drop_function`,
`Com_drop_index`,
`Com_drop_table`,
`Com_drop_user`,
`Com_execute_sql`,
`Com_flush`,
`Com_grant`,
`Com_ha_close`,
`Com_ha_open`,
`Com_ha_read`,
`Com_help`,
`Com_insert`,
`Com_insert_select`,
`Com_kill`,
`Com_load`,
`Com_load_master_data`,
`Com_load_master_table`,
`Com_lock_tables`,
`Com_optimize`,
`Com_preload_keys`,
`Com_prepare_sql`,
`Com_purge`,
`Com_purge_before_date`,
`Com_rename_table`,
`Com_repair`,
`Com_replace`,
`Com_replace_select`,
`Com_reset`,
`Com_restore_table`,
`Com_revoke`,
`Com_revoke_all`,
`Com_rollback`,
`Com_savepoint`,
`Com_select`,
`Com_set_option`,
`Com_show_binlog_events`,
`Com_show_binlogs`,
`Com_show_charsets`,
`Com_show_collations`,
`Com_show_column_types`,
`Com_show_create_db`,
`Com_show_create_table`,
`Com_show_databases`,
`Com_show_errors`,
`Com_show_fields`,
`Com_show_grants`,
`Com_show_innodb_status`,
`Com_show_keys`,
`Com_show_logs`,
`Com_show_master_status`,
`Com_show_ndb_status`,
`Com_show_new_master`,
`Com_show_open_tables`,
`Com_show_privileges`,
`Com_show_processlist`,
`Com_show_slave_hosts`,
`Com_show_slave_status`,
`Com_show_status`,
`Com_show_storage_engines`,
`Com_show_tables`,
`Com_show_triggers`,
`Com_show_variables`,
`Com_show_warnings`,
`Com_slave_start`,
`Com_slave_stop`,
`Com_stmt_close`,
`Com_stmt_execute`,
`Com_stmt_fetch`,
`Com_stmt_prepare`,
`Com_stmt_reset`,
`Com_stmt_send_long_data`,
`Com_truncate`,
`Com_unlock_tables`,
`Com_update`,
`Com_update_multi`,
`Com_xa_commit`,
`Com_xa_end`,
`Com_xa_prepare`,
`Com_xa_recover`,
`Com_xa_rollback`,
`Com_xa_start`,
`Compression`,
`Connections`,
`Created_tmp_disk_tables`,
`Created_tmp_files`,
`Created_tmp_tables`,
`Delayed_errors`,
`Delayed_insert_threads`,
`Delayed_writes`,
`Flush_commands`,
`Handler_commit`,
`Handler_delete`,
`Handler_discover`,
`Handler_prepare`,
`Handler_read_first`,
`Handler_read_key`,
`Handler_read_next`,
`Handler_read_prev`,
`Handler_read_rnd`,
`Handler_read_rnd_next`,
`Handler_rollback`,
`Handler_savepoint`,
`Handler_savepoint_rollback`,
`Handler_update`,
`Handler_write`,
`Innodb_buffer_pool_pages_data`,
`Innodb_buffer_pool_pages_dirty`,
`Innodb_buffer_pool_pages_flushed`,
`Innodb_buffer_pool_pages_free`,
`Innodb_buffer_pool_pages_misc`,
`Innodb_buffer_pool_pages_total`,
`Innodb_buffer_pool_read_ahead_rnd`,
`Innodb_buffer_pool_read_ahead_seq`,
`Innodb_buffer_pool_read_requests`,
`Innodb_buffer_pool_reads`,
`Innodb_buffer_pool_wait_free`,
`Innodb_buffer_pool_write_requests`,
`Innodb_data_fsyncs`,
`Innodb_data_pending_fsyncs`,
`Innodb_data_pending_reads`,
`Innodb_data_pending_writes`,
`Innodb_data_read`,
`Innodb_data_reads`,
`Innodb_data_writes`,
`Innodb_data_written`,
`Innodb_dblwr_pages_written`,
`Innodb_dblwr_writes`,
`Innodb_log_waits`,
`Innodb_log_write_requests`,
`Innodb_log_writes`,
`Innodb_os_log_fsyncs`,
`Innodb_os_log_pending_fsyncs`,
`Innodb_os_log_pending_writes`,
`Innodb_os_log_written`,
`Innodb_page_size`,
`Innodb_pages_created`,
`Innodb_pages_read`,
`Innodb_pages_written`,
`Innodb_row_lock_current_waits`,
`Innodb_row_lock_time`,
`Innodb_row_lock_time_avg`,
`Innodb_row_lock_time_max`,
`Innodb_row_lock_waits`,
`Innodb_rows_deleted`,
`Innodb_rows_inserted`,
`Innodb_rows_read`,
`Innodb_rows_updated`,
`Key_blocks_not_flushed`,
`Key_blocks_unused`,
`Key_blocks_used`,
`Key_read_requests`,
`Key_reads`,
`Key_write_requests`,
`Key_writes`,
`Last_query_cost`,
`Max_used_connections`,
`Not_flushed_delayed_rows`,
`Open_files`,
`Open_streams`,
`Open_tables`,
`Opened_tables`,
`Prepared_stmt_count`,
`Qcache_free_blocks`,
`Qcache_free_memory`,
`Qcache_hits`,
`Qcache_inserts`,
`Qcache_lowmem_prunes`,
`Qcache_not_cached`,
`Qcache_queries_in_cache`,
`Qcache_total_blocks`,
`Questions`,
`Rpl_status`,
`Select_full_join`,
`Select_full_range_join`,
`Select_range`,
`Select_range_check`,
`Select_scan`,
`Slave_open_temp_tables`,
`Slave_retried_transactions`,
`Slave_running`,
`Slow_launch_threads`,
`Slow_queries`,
`Sort_merge_passes`,
`Sort_range`,
`Sort_rows`,
`Sort_scan`,
`Table_locks_immediate`,
`Table_locks_waited`,
`Tc_log_max_pages_used`,
`Tc_log_page_size`,
`Tc_log_page_waits`,
`Threads_cached`,
`Threads_connected`,
`Threads_created`,
`Threads_running`,
`Uptime`,
`server_snmp_error_code`,
`server_mysql_error_code`,
`Slave_SQL_Running`,
`Slave_IO_Running`,
`Seconds_Behind_Master`,
`illegal_global_user`,
`illegal_grant_user`,
`illegal_remote_root`,
`illegal_user_nopass`,
`illegal_user_noname`,
`collection_time_elapse`,
`Creation_time`)

VALUES (
NULL, 
$varlist{'server_list_id'},
$varlist{'cnf_file'},
$varlist{'os_load_1'},
$varlist{'os_load_5'},
$varlist{'os_load_15'},
$varlist{'os_mem_total'},
$varlist{'os_mem_used'},
$varlist{'os_swap_total'},
$varlist{'os_swap_free'},
$varlist{'os_cpu_user'},
$varlist{'os_cpu_system'},
$varlist{'os_cpu_idle'},
$varlist{'queries_per_second'},
$varlist{'num_schema'},
$varlist{'num_tables'},
$varlist{'num_connections'},
$varlist{'length_data'},
$varlist{'length_index'},
$varlist{'engine_count_innodb'},
$varlist{'engine_count_myisam'},
$varlist{'engine_myisam_size_data'},
$varlist{'engine_myisam_size_index'},
$varlist{'engine_innodb_size_data'},
$varlist{'engine_innodb_size_index'},
$varlist{'auto_increment_increment'},
$varlist{'auto_increment_offset'},
$varlist{'automatic_sp_privileges'},
$varlist{'back_log'},
$varlist{'basedir'},
$varlist{'binlog_cache_size'},
$varlist{'bulk_insert_buffer_size'},
$varlist{'character_set_client'},
$varlist{'character_set_connection'},
$varlist{'character_set_database'},
$varlist{'character_set_filesystem'},
$varlist{'character_set_results'},
$varlist{'character_set_server'},
$varlist{'character_set_system'},
$varlist{'character_sets_dir'},
$varlist{'collation_connection'},
$varlist{'collation_database'},
$varlist{'collation_server'},
$varlist{'completion_type'},
$varlist{'concurrent_insert'},
$varlist{'connect_timeout'},
$varlist{'datadir'},
$varlist{'date_format'},
$varlist{'datetime_format'},
$varlist{'default_week_format'},
$varlist{'delay_key_write'},
$varlist{'delayed_insert_limit'},
$varlist{'delayed_insert_timeout'},
$varlist{'delayed_queue_size'},
$varlist{'div_precision_increment'},
$varlist{'keep_files_on_create'},
$varlist{'engine_condition_pushdown'},
$varlist{'expire_logs_days'},
$varlist{'flush'},
$varlist{'flush_time'},
$varlist{'ft_boolean_syntax'},
$varlist{'ft_max_word_len'},
$varlist{'ft_min_word_len'},
$varlist{'ft_query_expansion_limit'},
$varlist{'ft_stopword_file'},
$varlist{'group_concat_max_len'},
$varlist{'have_archive'},
$varlist{'have_bdb'},
$varlist{'have_blackhole_engine'},
$varlist{'have_compress'},
$varlist{'have_crypt'},
$varlist{'have_csv'},
$varlist{'have_dynamic_loading'},
$varlist{'have_example_engine'},
$varlist{'have_federated_engine'},
$varlist{'have_geometry'},
$varlist{'have_innodb'},
$varlist{'have_isam'},
$varlist{'have_merge_engine'},
$varlist{'have_ndbcluster'},
$varlist{'have_openssl'},
$varlist{'have_ssl'},
$varlist{'have_query_cache'},
$varlist{'have_raid'},
$varlist{'have_rtree_keys'},
$varlist{'have_symlink'},
$varlist{'hostname'},
$varlist{'init_connect'},
$varlist{'init_file'},
$varlist{'init_slave'},
$varlist{'innodb_additional_mem_pool_size'},
$varlist{'innodb_autoextend_increment'},
$varlist{'innodb_buffer_pool_awe_mem_mb'},
$varlist{'innodb_buffer_pool_size'},
$varlist{'innodb_checksums'},
$varlist{'innodb_commit_concurrency'},
$varlist{'innodb_concurrency_tickets'},
$varlist{'innodb_data_file_path'},
$varlist{'innodb_data_home_dir'},
$varlist{'innodb_adaptive_hash_index'},
$varlist{'innodb_doublewrite'},
$varlist{'innodb_fast_shutdown'},
$varlist{'innodb_file_io_threads'},
$varlist{'innodb_file_per_table'},
$varlist{'innodb_flush_log_at_trx_commit'},
$varlist{'innodb_flush_method'},
$varlist{'innodb_force_recovery'},
$varlist{'innodb_lock_wait_timeout'},
$varlist{'innodb_locks_unsafe_for_binlog'},
$varlist{'innodb_log_arch_dir'},
$varlist{'innodb_log_archive'},
$varlist{'innodb_log_buffer_size'},
$varlist{'innodb_log_file_size'},
$varlist{'innodb_log_files_in_group'},
$varlist{'innodb_log_group_home_dir'},
$varlist{'innodb_max_dirty_pages_pct'},
$varlist{'innodb_max_purge_lag'},
$varlist{'innodb_mirrored_log_groups'},
$varlist{'innodb_open_files'},
$varlist{'innodb_rollback_on_timeout'},
$varlist{'innodb_support_xa'},
$varlist{'innodb_sync_spin_loops'},
$varlist{'innodb_table_locks'},
$varlist{'innodb_thread_concurrency'},
$varlist{'innodb_thread_sleep_delay'},
$varlist{'innodb_read_ahead'},
$varlist{'innodb_ibuf_contract_const'},
$varlist{'innodb_ibuf_contract_burst'},
$varlist{'innodb_buf_flush_const'},
$varlist{'innodb_buf_flush_burst'},
$varlist{'interactive_timeout'},
$varlist{'join_buffer_size'},
$varlist{'key_buffer_size'},
$varlist{'key_cache_age_threshold'},
$varlist{'key_cache_block_size'},
$varlist{'key_cache_division_limit'},
$varlist{'language'},
$varlist{'large_files_support'},
$varlist{'large_page_size'},
$varlist{'large_pages'},
$varlist{'lc_time_names'},
$varlist{'license'},
$varlist{'local_infile'},
$varlist{'locked_in_memory'},
$varlist{'log'},
$varlist{'log_bin'},
$varlist{'log_bin_trust_function_creators'},
$varlist{'log_error'},
$varlist{'log_queries_not_using_indexes'},
$varlist{'log_slave_updates'},
$varlist{'log_slow_queries'},
$varlist{'log_slow_filter'},
$varlist{'log_slow_verbosity'},
$varlist{'log_warnings'},
$varlist{'long_query_time'},
$varlist{'low_priority_updates'},
$varlist{'lower_case_file_system'},
$varlist{'lower_case_table_names'},
$varlist{'max_allowed_packet'},
$varlist{'max_binlog_cache_size'},
$varlist{'max_binlog_size'},
$varlist{'max_connect_errors'},
$varlist{'max_connections'},
$varlist{'max_delayed_threads'},
$varlist{'max_error_count'},
$varlist{'max_heap_table_size'},
$varlist{'max_insert_delayed_threads'},
$varlist{'max_join_size'},
$varlist{'max_length_for_sort_data'},
$varlist{'max_prepared_stmt_count'},
$varlist{'max_relay_log_size'},
$varlist{'max_seeks_for_key'},
$varlist{'max_sort_length'},
$varlist{'max_sp_recursion_depth'},
$varlist{'max_tmp_tables'},
$varlist{'max_user_connections'},
$varlist{'max_write_lock_count'},
$varlist{'min_examined_row_limit'},
$varlist{'multi_range_count'},
$varlist{'myisam_data_pointer_size'},
$varlist{'myisam_max_sort_file_size'},
$varlist{'myisam_recover_options'},
$varlist{'myisam_repair_threads'},
$varlist{'myisam_sort_buffer_size'},
$varlist{'myisam_stats_method'},
$varlist{'net_buffer_length'},
$varlist{'net_read_timeout'},
$varlist{'net_retry_count'},
$varlist{'net_write_timeout'},
$varlist{'new'},
$varlist{'old_passwords'},
$varlist{'open_files_limit'},
$varlist{'optimizer_prune_level'},
$varlist{'optimizer_search_depth'},
$varlist{'pid_file'},
$varlist{'port'},
$varlist{'preload_buffer_size'},
$varlist{'protocol_version'},
$varlist{'query_alloc_block_size'},
$varlist{'query_cache_limit'},
$varlist{'query_cache_min_res_unit'},
$varlist{'query_cache_size'},
$varlist{'query_cache_type'},
$varlist{'query_cache_wlock_invalidate'},
$varlist{'query_prealloc_size'},
$varlist{'range_alloc_block_size'},
$varlist{'log_slow_rate_limit'},
$varlist{'read_buffer_size'},
$varlist{'read_only'},
$varlist{'read_rnd_buffer_size'},
$varlist{'relay_log'},
$varlist{'relay_log_index'},
$varlist{'relay_log_info_file'},
$varlist{'relay_log_purge'},
$varlist{'relay_log_space_limit'},
$varlist{'rpl_recovery_rank'},
$varlist{'secure_auth'},
$varlist{'secure_file_priv'},
$varlist{'server_id'},
$varlist{'skip_external_locking'},
$varlist{'skip_networking'},
$varlist{'skip_show_database'},
$varlist{'slave_compressed_protocol'},
$varlist{'slave_load_tmpdir'},
$varlist{'slave_net_timeout'},
$varlist{'slave_skip_errors'},
$varlist{'slave_transaction_retries'},
$varlist{'slow_launch_time'},
$varlist{'socket'},
$varlist{'sort_buffer_size'},
$varlist{'sql_big_selects'},
$varlist{'sql_mode'},
$varlist{'sql_notes'},
$varlist{'sql_warnings'},
$varlist{'ssl_ca'},
$varlist{'ssl_capath'},
$varlist{'ssl_cert'},
$varlist{'ssl_cipher'},
$varlist{'ssl_key'},
$varlist{'storage_engine'},
$varlist{'sync_binlog'},
$varlist{'sync_frm'},
$varlist{'system_time_zone'},
$varlist{'table_cache'},
$varlist{'table_lock_wait_timeout'},
$varlist{'table_type'},
$varlist{'thread_cache_size'},
$varlist{'thread_stack'},
$varlist{'time_format'},
$varlist{'time_zone'},
$varlist{'timed_mutexes'},
$varlist{'tmp_table_size'},
$varlist{'tmpdir'},
$varlist{'transaction_alloc_block_size'},
$varlist{'transaction_prealloc_size'},
$varlist{'tx_isolation'},
$varlist{'updatable_views_with_limit'},
$varlist{'version'},
$varlist{'version_comment'},
$varlist{'version_compile_machine'},
$varlist{'version_compile_os'},
$varlist{'wait_timeout'},
$varlist{'Aborted_clients'},
$varlist{'Aborted_connects'},
$varlist{'Binlog_cache_disk_use'},
$varlist{'Binlog_cache_use'},
$varlist{'Bytes_received'},
$varlist{'Bytes_sent'},
$varlist{'Com_admin_commands'},
$varlist{'Com_alter_db'},
$varlist{'Com_alter_table'},
$varlist{'Com_analyze'},
$varlist{'Com_backup_table'},
$varlist{'Com_begin'},
$varlist{'Com_call_procedure'},
$varlist{'Com_change_db'},
$varlist{'Com_change_master'},
$varlist{'Com_check'},
$varlist{'Com_checksum'},
$varlist{'Com_commit'},
$varlist{'Com_create_db'},
$varlist{'Com_create_function'},
$varlist{'Com_create_index'},
$varlist{'Com_create_table'},
$varlist{'Com_create_user'},
$varlist{'Com_dealloc_sql'},
$varlist{'Com_delete'},
$varlist{'Com_delete_multi'},
$varlist{'Com_do'},
$varlist{'Com_drop_db'},
$varlist{'Com_drop_function'},
$varlist{'Com_drop_index'},
$varlist{'Com_drop_table'},
$varlist{'Com_drop_user'},
$varlist{'Com_execute_sql'},
$varlist{'Com_flush'},
$varlist{'Com_grant'},
$varlist{'Com_ha_close'},
$varlist{'Com_ha_open'},
$varlist{'Com_ha_read'},
$varlist{'Com_help'},
$varlist{'Com_insert'},
$varlist{'Com_insert_select'},
$varlist{'Com_kill'},
$varlist{'Com_load'},
$varlist{'Com_load_master_data'},
$varlist{'Com_load_master_table'},
$varlist{'Com_lock_tables'},
$varlist{'Com_optimize'},
$varlist{'Com_preload_keys'},
$varlist{'Com_prepare_sql'},
$varlist{'Com_purge'},
$varlist{'Com_purge_before_date'},
$varlist{'Com_rename_table'},
$varlist{'Com_repair'},
$varlist{'Com_replace'},
$varlist{'Com_replace_select'},
$varlist{'Com_reset'},
$varlist{'Com_restore_table'},
$varlist{'Com_revoke'},
$varlist{'Com_revoke_all'},
$varlist{'Com_rollback'},
$varlist{'Com_savepoint'},
$varlist{'Com_select'},
$varlist{'Com_set_option'},
$varlist{'Com_show_binlog_events'},
$varlist{'Com_show_binlogs'},
$varlist{'Com_show_charsets'},
$varlist{'Com_show_collations'},
$varlist{'Com_show_column_types'},
$varlist{'Com_show_create_db'},
$varlist{'Com_show_create_table'},
$varlist{'Com_show_databases'},
$varlist{'Com_show_errors'},
$varlist{'Com_show_fields'},
$varlist{'Com_show_grants'},
$varlist{'Com_show_innodb_status'},
$varlist{'Com_show_keys'},
$varlist{'Com_show_logs'},
$varlist{'Com_show_master_status'},
$varlist{'Com_show_ndb_status'},
$varlist{'Com_show_new_master'},
$varlist{'Com_show_open_tables'},
$varlist{'Com_show_privileges'},
$varlist{'Com_show_processlist'},
$varlist{'Com_show_slave_hosts'},
$varlist{'Com_show_slave_status'},
$varlist{'Com_show_status'},
$varlist{'Com_show_storage_engines'},
$varlist{'Com_show_tables'},
$varlist{'Com_show_triggers'},
$varlist{'Com_show_variables'},
$varlist{'Com_show_warnings'},
$varlist{'Com_slave_start'},
$varlist{'Com_slave_stop'},
$varlist{'Com_stmt_close'},
$varlist{'Com_stmt_execute'},
$varlist{'Com_stmt_fetch'},
$varlist{'Com_stmt_prepare'},
$varlist{'Com_stmt_reset'},
$varlist{'Com_stmt_send_long_data'},
$varlist{'Com_truncate'},
$varlist{'Com_unlock_tables'},
$varlist{'Com_update'},
$varlist{'Com_update_multi'},
$varlist{'Com_xa_commit'},
$varlist{'Com_xa_end'},
$varlist{'Com_xa_prepare'},
$varlist{'Com_xa_recover'},
$varlist{'Com_xa_rollback'},
$varlist{'Com_xa_start'},
$varlist{'Compression'},
$varlist{'Connections'},
$varlist{'Created_tmp_disk_tables'},
$varlist{'Created_tmp_files'},
$varlist{'Created_tmp_tables'},
$varlist{'Delayed_errors'},
$varlist{'Delayed_insert_threads'},
$varlist{'Delayed_writes'},
$varlist{'Flush_commands'},
$varlist{'Handler_commit'},
$varlist{'Handler_delete'},
$varlist{'Handler_discover'},
$varlist{'Handler_prepare'},
$varlist{'Handler_read_first'},
$varlist{'Handler_read_key'},
$varlist{'Handler_read_next'},
$varlist{'Handler_read_prev'},
$varlist{'Handler_read_rnd'},
$varlist{'Handler_read_rnd_next'},
$varlist{'Handler_rollback'},
$varlist{'Handler_savepoint'},
$varlist{'Handler_savepoint_rollback'},
$varlist{'Handler_update'},
$varlist{'Handler_write'},
$varlist{'Innodb_buffer_pool_pages_data'},
$varlist{'Innodb_buffer_pool_pages_dirty'},
$varlist{'Innodb_buffer_pool_pages_flushed'},
$varlist{'Innodb_buffer_pool_pages_free'},
$varlist{'Innodb_buffer_pool_pages_misc'},
$varlist{'Innodb_buffer_pool_pages_total'},
$varlist{'Innodb_buffer_pool_read_ahead_rnd'},
$varlist{'Innodb_buffer_pool_read_ahead_seq'},
$varlist{'Innodb_buffer_pool_read_requests'},
$varlist{'Innodb_buffer_pool_reads'},
$varlist{'Innodb_buffer_pool_wait_free'},
$varlist{'Innodb_buffer_pool_write_requests'},
$varlist{'Innodb_data_fsyncs'},
$varlist{'Innodb_data_pending_fsyncs'},
$varlist{'Innodb_data_pending_reads'},
$varlist{'Innodb_data_pending_writes'},
$varlist{'Innodb_data_read'},
$varlist{'Innodb_data_reads'},
$varlist{'Innodb_data_writes'},
$varlist{'Innodb_data_written'},
$varlist{'Innodb_dblwr_pages_written'},
$varlist{'Innodb_dblwr_writes'},
$varlist{'Innodb_log_waits'},
$varlist{'Innodb_log_write_requests'},
$varlist{'Innodb_log_writes'},
$varlist{'Innodb_os_log_fsyncs'},
$varlist{'Innodb_os_log_pending_fsyncs'},
$varlist{'Innodb_os_log_pending_writes'},
$varlist{'Innodb_os_log_written'},
$varlist{'Innodb_page_size'},
$varlist{'Innodb_pages_created'},
$varlist{'Innodb_pages_read'},
$varlist{'Innodb_pages_written'},
$varlist{'Innodb_row_lock_current_waits'},
$varlist{'Innodb_row_lock_time'},
$varlist{'Innodb_row_lock_time_avg'},
$varlist{'Innodb_row_lock_time_max'},
$varlist{'Innodb_row_lock_waits'},
$varlist{'Innodb_rows_deleted'},
$varlist{'Innodb_rows_inserted'},
$varlist{'Innodb_rows_read'},
$varlist{'Innodb_rows_updated'},
$varlist{'Key_blocks_not_flushed'},
$varlist{'Key_blocks_unused'},
$varlist{'Key_blocks_used'},
$varlist{'Key_read_requests'},
$varlist{'Key_reads'},
$varlist{'Key_write_requests'},
$varlist{'Key_writes'},
$varlist{'Last_query_cost'},
$varlist{'Max_used_connections'},
$varlist{'Not_flushed_delayed_rows'},
$varlist{'Open_files'},
$varlist{'Open_streams'},
$varlist{'Open_tables'},
$varlist{'Opened_tables'},
$varlist{'Prepared_stmt_count'},
$varlist{'Qcache_free_blocks'},
$varlist{'Qcache_free_memory'},
$varlist{'Qcache_hits'},
$varlist{'Qcache_inserts'},
$varlist{'Qcache_lowmem_prunes'},
$varlist{'Qcache_not_cached'},
$varlist{'Qcache_queries_in_cache'},
$varlist{'Qcache_total_blocks'},
$varlist{'Questions'},
$varlist{'Rpl_status'},
$varlist{'Select_full_join'},
$varlist{'Select_full_range_join'},
$varlist{'Select_range'},
$varlist{'Select_range_check'},
$varlist{'Select_scan'},
$varlist{'Slave_open_temp_tables'},
$varlist{'Slave_retried_transactions'},
$varlist{'Slave_running'},
$varlist{'Slow_launch_threads'},
$varlist{'Slow_queries'},
$varlist{'Sort_merge_passes'},
$varlist{'Sort_range'},
$varlist{'Sort_rows'},
$varlist{'Sort_scan'},
$varlist{'Table_locks_immediate'},
$varlist{'Table_locks_waited'},
$varlist{'Tc_log_max_pages_used'},
$varlist{'Tc_log_page_size'},
$varlist{'Tc_log_page_waits'},
$varlist{'Threads_cached'},
$varlist{'Threads_connected'},
$varlist{'Threads_created'},
$varlist{'Threads_running'},
$varlist{'Uptime'},
$varlist{'server_snmp_error_code'},
$varlist{'server_mysql_error_code'},
$varlist{'Slave_SQL_Running'},
$varlist{'Slave_IO_Running'},
$varlist{'Seconds_Behind_Master'},
$varlist{'illegal_global_user'},
$varlist{'illegal_grant_user'},
$varlist{'illegal_remote_root'},
$varlist{'illegal_user_nopass'},
$varlist{'illegal_user_noname'},
$varlist{'collection_time_elapse'},
NOW()
);";

    my $dbh = DBI->connect(
        "DBI:mysql:$mysql_w_db:$mysql_w_host",
        $mysql_w_user,
        $mysql_w_pass,
	{
            PrintError => 0,
            RaiseError => 0
	}
	) or error_report("$DBI::errstr");
    
    my $sth = $dbh->prepare($sql0) or error_report("$DBI::errstr");
    $sth->execute or error_report("$DBI::errstr");
    $sth->finish;
    $dbh->disconnect;
}

#populate all of the variables in the query list with their values from the xml file processing below
sub parse_data {
    my $dbh = DBI->connect(
        "DBI:mysql:$mysql_r_db:$mysql_r_host",
        $mysql_r_user,
        $mysql_r_pass,
	{
            PrintError => 0,
            RaiseError => 0
	}
	) or error_report("$DBI::errstr");
    
    my $file = $_[0];
    my $server_id = $_[1];
    my $active_status = $_[2];
    my $updated_status;
    my $parser = XML::Parser->new(ErrorContext => 2, Style => "Tree");
    my $xso = XML::SimpleObject->new( $parser->parsefile($file));
    
    foreach my $server ($xso->child('kontrollbase')->children('server')) {    
	$h = $server->attribute('hostname');
        debug_report("$h [starting]");

	# we get the time of the dataset 
        foreach my $datetime($server->child('datetime')) {
	    my $dt = $datetime->value;
            debug_report("$h xml datetime = $dt");
	    $varlist{'datetime'} = $datetime->value;
        }
	
	# we find errors!
	if($server->child('error')) {
	    foreach my $error($server->child('error')) {
		my $error_type = $error->attribute('type');
		my $error_value = $dbh->quote($error->value);

		debug_report("error = $error_value");
		debug_report("error type = $error_type");
		
		if($error_type eq "snmp") {
		    $varlist{'server_snmp_error_code'} = $error_value;
		}
		if($error_type eq "mysql") {
                    $varlist{'server_mysql_error_code'} = $error_value;
                }
		if($error_type eq "mysql-connection-error") {
                    $varlist{'server_mysql_error_code'} = $error_value;
		    $updated_status = 2;
                    modify_active_status($server_id,$active_status,$updated_status);
	            error_report("mysql connection error occured, setting status of $h to inactive");	
	    }
	    }
	}

	# we assign values to hash
	while (my($key,$value) = each(%varlist)) {
	    foreach my $item($server->child('item')) {
		my $item_name = $item->attribute('name');
		my $item_value = '0';		
		if($item->value) { $item_value = $item->value; }
		if($item_name eq $key) {
		    $value = $dbh->quote($item_value);
		    #print "$key => $value: ";
		    $varlist{$key} = $value;
		    #print $varlist{$key}."\n";
		}		
	    }
	}

	#do some variable cleaning and math for queries/sec
	my $Questions = $varlist{'Questions'};
	my $Uptime = $varlist{'Uptime'};
	$Questions =~ s/'//g;
	$Uptime =~ s/'//g;
	my $qsec = ($Questions/$Uptime);
	$varlist{'queries_per_second'}=$qsec;
	my $os_mem_total = $varlist{'os_mem_total'};
	debug_report("$h queries/sec=$qsec");
	debug_report("total ram=$os_mem_total");

	#print compute time from xml
	my $time = $varlist{'collection_time_elapse'};
	$time =~ s/'//g;
	debug_report("$h Collection time: $time seconds");
    }
    $dbh->disconnect;
    debug_report("$h [success]");
}

sub error_report {
    my $err0 = $_[0];
    my $errtime =  strftime "%Y-%m-%d %H:%M:%S", localtime;
    my $err1 = $errtime." | ERROR | stats-gather-5.0.x_linux-x86: ".$err0."\n";
    print $err1;
    sysopen(FILE, $error_log, O_RDWR|O_EXCL|O_CREAT, 0600);    
    open FILE, ">>$error_log" or die $!; 
    print FILE $err1; 
    close FILE;   
    debug_report($err0);
    exit 1;
}

sub debug_report {
    my $note = $_[0];
    my $debugtime =  strftime "%Y-%m-%d %H:%M:%S", localtime;
    $note = $debugtime." | DEBUG | stats-gather-5.0.x_linux-x86: ".$note."\n";
    print $note;
    sysopen(FILE, $debug_log, O_RDWR|O_EXCL|O_CREAT, 0600);
    open FILE, ">>$debug_log" or die $!;
    print FILE $note;
    close FILE;
}

sub modify_active_status {

     my $server_id = $_[0];
     my $active_status = $_[1];
     my $updated_status = $_[2];
     my $sql1;

     my $dbh = DBI->connect(
     "DBI:mysql:$mysql_w_db:$mysql_w_host",
     $mysql_w_user,
     $mysql_w_pass,
        {
            PrintError => 0,
            RaiseError => 0
        }
        ) or error_report("$DBI::errstr");

    if (($active_status == 1) && ($updated_status == 2)) {
    $sql1 = "UPDATE server_list set active = '2' where id = $server_id;"; 
    debug_report("connection with $h could not be established, updating database, setting active = '2'") 
   }
    elsif (($active_status == 2) && ($updated_status == 1)) {
    $sql1 = "UPDATE server_list set active = '1' where id = $server_id;";
    debug_report("connection with $h reestablished, updating database, setting active = '1'") 
    }
    elsif ($active_status == $updated_status) {
    $dbh->disconnect;
    error_report("connection status of $h remains unchanged, no need to update database");
    }

    my $sth = $dbh->prepare($sql1) or error_report("$DBI::errstr");
    $sth->execute or error_report("$DBI::errstr");
    $sth->finish;
    $dbh->disconnect;
    exit 1;

}
config_connect($config);
my $xml_file = $ARGV[0];
my $server_id = $ARGV[1];
my $active_status = $ARGV[2];
my $updated_status = 1;

$varlist{'server_list_id'} = $ARGV[1];
if(!$xml_file) { error_report("No input file specified. Exiting.");}
if(!$ARGV[1]) { error_report("No server_id value specified. Exiting.");}
debug_report("## process start");
parse_data($xml_file,$server_id,$active_status);
insert_data() or error_report("$h [fail]");
if ($active_status == 2) { modify_active_status( $server_id,$active_status,$updated_status);}


